<?php
$controls = new LiteCacheControls();     
        
if ($controls->is_action('save')) {
    $controls->options = stripslashes_deep($_POST['options']);
    
    $controls->options['parsed_urls'] = array();
    $urls = explode("\n", str_replace(array("\n", "\r", " "), "\n", $controls->options['urls']));
    foreach ($urls as &$uri)
    {
        $uri = trim($uri);
        if ($uri == '') continue;
        if ($uri[0] != '/') $uri = '/' . $uri;
        $controls->options['parsed_urls'][] = $uri;
    }    
    
    $controls->options['folder'] = trim($controls->options['folder']);
    if (!empty($controls->options['folder'])) $controls->options['folder'] = untrailingslashit($controls->options['folder']);
    
    if (empty($controls->options['max_age']) || !is_numeric($controls->options['max_age'])) $controls->options['max_age'] = 0;
    
    /*
    $controls->options['parsed_cookies'] = array();
    $cookies = explode("\n", str_replace(array("\n", "\r", " "), "\n", $controls->options['cookies']));
    foreach ($cookies as &$cookie)
    {
        $cookie = trim($cookie);
        if ($cookie == '') continue;
        $controls->options['parsed_cookies'][] = $uri;
    }    
    */
    
    if (empty($controls->options['agents'])) $controls->options['agents'] = LiteCache::AGENTS;
    update_option('lite-cache', $controls->options);
    
    $advanced_cache = file_get_contents(dirname(__FILE__) . '/advanced-cache.php');
    $advanced_cache = str_replace('_AGENTS_', $controls->options['agents'], $advanced_cache);
    $advanced_cache = str_replace('_MOBILE_', $controls->options['mobile'], $advanced_cache);
    $advanced_cache = str_replace('_FOLDER_', $lite_cache->get_folder(), $advanced_cache);
    $advanced_cache = str_replace('_MAX_AGE_', $controls->options['max_age'], $advanced_cache);
    $r = file_put_contents(WP_CONTENT_DIR . '/advanced-cache.php', $advanced_cache);
    if ($r == false) {
        $controls->errors = '<code>wp-content/advanced-cache.php</code>ファイルに書き込めません。ファイルまたはフォルダのアクセス権を確認してください。';
    }
}

if ($controls->is_action('clean')) {
    $folder = $lite_cache->get_folder();
    $lite_cache->remove_dir($folder . '');
}

if ($controls->is_action('clean-home')) {
    $folder = $lite_cache->get_folder();
    @unlink($folder . '/index.html');
    @unlink($folder . '/index.html.gz');
    @unlink($folder . '/index-mobile.html');
    @unlink($folder . '/index-mobile.html.gz');
    @unlink($folder . '/index-mobile-user.html');
    @unlink($folder . '/index-mobile-user.html.gz');
    @unlink($folder . '/index-user.html');
    @unlink($folder . '/index-user.html.gz');
    @unlink($folder . '/robots.txt');
    $lite_cache->remove_dir($folder . '/feed/');
    $lite_cache->remove_dir($folder . '/page/');
    $base = get_option('category_base');
    if (empty($base)) $base = 'category';
    $lite_cache->remove_dir($folder . '/' . $base . '/');

    $base = get_option('tag_base');
    if (empty($base)) $base = 'tag';
    $lite_cache->remove_dir($folder . '/' . $base . '/');

    $lite_cache->remove_dir($folder . '/type/');

    $lite_cache->remove_dir($folder . '/' . date('Y') . '/');
}


if ($controls->is_action('size')) { 
    $folder = $lite_cache->get_folder();
    $controls->messages = 'Cache size: ' . size_format((lc_size($folder . '/')));
}

function lc_size($dir) {
    $files = glob($dir . '*', GLOB_MARK);
    $size = 0;
    foreach ($files as &$file) {
        if (substr($file, -1) == '/')
            $size += lc_size($file);
        else
            $size += @filesize($file);
    }
    return $size;
}

if ($controls->options == null) $controls->options = get_option('lite-cache');

// For installation that does not create the directory on activation
wp_mkdir_p($lite_cache->get_folder());   

?>

<div class="wrap">
  <h2>Lite Cache</h2>
  
  <?php if (!defined('WP_CACHE') || !WP_CACHE) { ?>
    <div class="error">
      <p>
        <strong>wp-config.phpの&lt;?phpの次の行に<code>define('WP_CACHE', true);</code>を入力する必要があります。すこし制限されますが、指定した.htaccessにコードを追加した場合は入力しなくても構いません。</strong>
      </p>
    </div>
  <?php } ?>

  <?php if (filemtime(WP_CONTENT_DIR . '/advanced-cache.php') < filemtime(dirname(__FILE__) . '/advanced-cache.php')) { ?>
    <div class="error">
      <p>
        いくつかのファイルが更新されなければならないので、設定を更新する必要があります。
      </p>
    </div>
  <?php } ?>    

  <?php if (!is_dir($lite_cache->get_folder())) { ?>
    <div class="error">
      <p>
        Lite Cacheは<code><?php $lite_cache->get_folder(); ?></code>フォルダーを作成あるいは読み込みできませんでした。手動で作成するか書き込み・読み込みが出来る権限に(普通は777)してください。
      </p>
    </div>
  <?php } ?>    

  <?php if (get_option('permalink_structure') == '') { ?>
    <div class="error"><p>Lite Cacheを動作させるには異なるパーマリンク構造を選択する必要があります。</p></div>
  <?php } ?>


  <?php $controls->show_messages(); ?>
  <?php $controls->show_errors(); ?>

  <iframe src="http://frames.satollo.net/lite-cache.php" frameborder="0" width="100%" height="80" style="border: 0"></iframe>
  <br />

  <form method="post" action="">
    <?php $controls->init(); ?>


    <p>なにか疑問があったら<a href="http://www.satollo.net/plugins/lite-cache" target="_tab">Lite Cache オフィシャルページ</a>で答えます。 (新しいタブで開きます)</p>

    <p>
      <?php $controls->button('clean', 'キャッシュをクリアする'); ?>
      <?php $controls->button('clean-home', 'アーカイブ・フィード・ホームのみキャッシュをクリアする'); ?>
      <?php $controls->button('size', 'キャッシュサイズを表示'); ?>
    </p>
    
    <div id="tabs">
      <ul>
        <li><a href="#tabs-1">設定</a></li>
        <li><a href="#tabs-2">携帯</a></li>
        <li><a href="#tabs-3">.htaccess</a></li>
        <li><a href="#tabs-4">高度</a></li>
      </ul>

      <div id="tabs-1">  

        <table class="form-table">
          <!--
          <tr>
              <th>Cache feed too?</th>
              <td>
          <?php $controls->checkbox('feed'); ?>
              </td>
          </tr>             
          -->
          <tr>
              <th>キャッシュの有効時間</th>
              <td>
                <?php $controls->text('max_age'); ?> 時間
                <div class="hints">「0」で永遠に有効になります。</div>
              </td>
          </tr>             
          
          <tr>
            <th>ログインユーザーにもキャッシュを有効にする</th>
            <td>
              <?php $controls->checkbox('user'); ?> 推奨しません！
              <div class="hints">管理者の場合にキャッシュを無効にしたい場合は<a href="http://www.satollo.net/plugins/lite-cache">こちら</a>をご覧ください。</div>
            </td>
          </tr> 
          <tr>
            <th>キャッシュから除外するURL</th>
            <td>
              <?php $controls->textarea('urls'); ?>
              <div class="hints">1行につき1つです。</div>
            </td>
          </tr> 
          <!--
          <tr>
              <th>Cookies that "bypass" the cache</th>
              <td>
                <?php $controls->textarea('coockies'); ?>
                <div class="hints">One per line</div>
              </td>
          </tr> 
          -->
        </table>

      </div>
      <div id="tabs-2"> 
        <table class="form-table">
          <!--
          <tr>
              <th>Cache feed too?</th>
              <td>
                <?php $controls->checkbox('feed'); ?>
              </td>
          </tr>             
          -->
          <tr>
            <th>モード</th>
            <td>
              <?php $controls->select('mobile', array(0 => '[無効]携帯を検出しない', 1 => '[有効]携帯を検出、別々のキャッシュを使用する', 2 => '[有効]携帯デバイスを検出、キャッシュを使用しない')); ?>
              <div class="hints">
                トラフィックが少ないときはキャッシュを有効にしなくてかまいません。
              </div>
            </td>
          </tr>
          <tr>
            <th>携帯テーマ</th>
            <td>
              <?php
              $themes = get_themes();
              $list = array('' => '現在のテーマを使用');
              foreach ($themes as $key => $data)
                $list[$key] = $key;
              ?>
              <?php $controls->select('theme', $list); ?>
              <div class="hints">
                携帯のために現在のテーマを使用する場合、他のプラグインが携帯のためのコンテンツやレイアウトを作成する場合、「携帯を検出、キャッシュを利用する」を選択する必要があります。
              </div>
            </td>
          </tr>
          <tr>
            <th>携帯ユーザーエージェント</th>
            <td>
              <?php $controls->text('agents', 80); ?>
              <div class="hints">小文字にして、パイプ(|)で区切ってください。スペース( 　)やアポストロフィ(&#39;)は使用しないでください。下記が初期値です:<br />
                <code><?php echo LiteCache::AGENTS; ?></code></div>
            </td>
          </tr>             
        </table>
      </div>

      <div id="tabs-3">  

        <p>
          このモードはブログが.htaccessを使用している場合のみ動作します。# BEGIN WordPressより前に以下のコードを追加・変更してください。このモードではログインユーザーに対するキャッシュ設定をWordPressから変更できません。

        </p>
        <p><strong>注意</strong>:ログインユーザーに対するキャッシュ設定を変更した場合はこのコードを再度確認して、変更してください！</p>

        <?php
        $home_root = parse_url(get_option('home'), PHP_URL_PATH);
        $home_root = trailingslashit($home_root);
        ?>
        <pre style="border: 1px solid #999; background-color: #fff; padding: 15px; font-family: monospace; font-size: 11px">
&lt;FilesMatch "\.html\.gz$"&gt;
ForceType text/html
&lt;/FilesMatch&gt;
AddEncoding gzip .gz
AddType text/html .gz
AddDefaultCharset UTF-8

RewriteEngine On
RewriteBase <?php echo $home_root; ?>

<?php if (false && $controls->options['user'] == 1) { ?>
RewriteCond %{HTTP:Cookie} !^.*cache_disable.*$
RewriteCond %{HTTP:Accept-Encoding} gzip
RewriteCond %{HTTP:Cookie} ^.*wordpress_logged_in.*$
RewriteCond %{HTTP:Cookie} !^.*wp-postpass_.*$
RewriteCond %{REQUEST_METHOD} !=POST
RewriteCond %{QUERY_STRING} !.+
<!--RewriteCond <?php echo ABSPATH; ?>wp-content/cache/lite-cache%{REQUEST_URI}/index-user.html.gz -f-->
RewriteCond <?php echo $lite_cache->get_folder(); ?>%{REQUEST_URI}/index-user.html.gz -f
RewriteRule ^(.*) wp-content/cache/lite-cache/$1/index-user.html.gz [L]

RewriteCond %{HTTP:Cookie} !^.*cache_disable.*$
RewriteCond %{HTTP:Cookie} ^.*wordpress_logged_in.*$
RewriteCond %{HTTP:Cookie} !^.*wp-postpass_.*$
RewriteCond %{REQUEST_METHOD} !=POST
RewriteCond %{QUERY_STRING} !.+
<!--RewriteCond <?php echo ABSPATH; ?>wp-content/cache/lite-cache%{REQUEST_URI}/index-user.html -f-->
RewriteCond <?php echo $lite_cache->get_folder(); ?>%{REQUEST_URI}/index-user.html -f
RewriteRule ^(.*) wp-content/cache/lite-cache/$1/index-user.html [L]
<?php } ?>

# Mobile gzip
RewriteCond %{HTTP:Cookie} !^.*cache_disable.*$
RewriteCond %{HTTP:Accept-Encoding} gzip
RewriteCond %{HTTP:Cookie} !^.*(wordpress_logged_in_|wp-postpass_).*$
RewriteCond %{HTTP_USER_AGENT} ^.*(<?php echo $options['mobile']; ?>).*$ [NC]
RewriteCond %{REQUEST_METHOD} !=POST
RewriteCond %{QUERY_STRING} !.+
<!--RewriteCond <?php echo ABSPATH; ?>wp-content/cache/lite-cache%{REQUEST_URI}/index-mobile.html.gz -f-->
RewriteCond <?php echo $lite_cache->get_folder(); ?>%{REQUEST_URI}/index-mobile.html.gz -f
RewriteRule ^(.*) wp-content/cache/lite-cache/$1/index-mobile.html.gz [L]

# Mobile
RewriteCond %{HTTP:Cookie} !^.*cache_disable.*$
RewriteCond %{HTTP:Cookie} !^.*(wordpress_logged_in_|wp-postpass_).*$
RewriteCond %{HTTP_USER_AGENT} ^.*(<?php echo $options['mobile']; ?>).*$ [NC]
RewriteCond %{REQUEST_METHOD} !=POST
RewriteCond %{QUERY_STRING} !.+
<!--RewriteCond <?php echo ABSPATH; ?>wp-content/cache/lite-cache%{REQUEST_URI}/index-mobile.html -f-->
RewriteCond <?php echo $lite_cache->get_folder(); ?>%{REQUEST_URI}/index-mobile.html -f
RewriteRule ^(.*) wp-content/cache/lite-cache/$1/index-mobile.html [L]

# Standard gzip
RewriteCond %{HTTP:Cookie} !^.*cache_disable.*$
RewriteCond %{HTTP:Accept-Encoding} gzip
RewriteCond %{HTTP:Cookie} !^.*(wordpress_logged_in_|wp-postpass_).*$
RewriteCond %{REQUEST_METHOD} !=POST
RewriteCond %{QUERY_STRING} !.+
<!--RewriteCond <?php echo ABSPATH; ?>wp-content/cache/lite-cache%{REQUEST_URI}/index.html.gz -f-->
RewriteCond <?php echo $lite_cache->get_folder(); ?>%{REQUEST_URI}/index.html.gz -f-->
RewriteRule ^(.*) wp-content/cache/lite-cache/$1/index.html.gz [L]

# Standard
RewriteCond %{HTTP:Cookie} !^.*cache_disable.*$
RewriteCond %{HTTP:Cookie} !^.*(wordpress_logged_in_|wp-postpass_).*$
RewriteCond %{REQUEST_METHOD} !=POST
RewriteCond %{QUERY_STRING} !.+
<!--RewriteCond <?php echo ABSPATH; ?>wp-content/cache/lite-cache%{REQUEST_URI}/index.html -f-->
RewriteCond <?php echo $lite_cache->get_folder(); ?>%{REQUEST_URI}/index.html -f
RewriteRule ^(.*) wp-content/cache/lite-cache/$1/index.html [L]
        </pre>
      </div>
      
      <div id="tabs-4"> 
        <table class="form-table">
          <tr>
              <th>最近何日までの投稿をキャッシュするか</th>
              <td>
                <?php $controls->text('newer_than_days'); ?> 日前までキャッシュ
                <p>
                  古い記事に対してのトラフィックが低い場合に便利です。ディスク容量が削減できます。
                </p>
              </td>
          </tr>             
          <tr>
              <th>キャッシュ保存フォルダ</th>
              <td>
                <?php $controls->text('folder', 70); ?> (ディスクからのフルパッチ)
                <p>
                  空白で初期値になります。キャッシュフォルダを変更した場合、.htaccessモードは動作しません。初期値のままにして<code>wp-content/cache/lite-cache -&gt; [your folder]</code>にシンボリックリンクを作成して他のフォルダにキャッシュを置くこともできます。 このブログは<code><?php echo ABSPATH; ?></code>より上にあります。間違えた設定にするとブログを壊してしまうことがあります。
              </td>
          </tr>             
        </table>
      </div>
          
    </div>
    <p>
      <?php $controls->button('save', '設定を保存'); ?>
    </p>

  </form>
</div>

<?php
class LiteCacheControls {

    var $options = null;
    var $errors = null;
    var $messages = null;

    function is_action($action=null) {
        if ($action == null) return !empty($_REQUEST['act']);
        if (empty($_REQUEST['act'])) return false;
        if ($_REQUEST['act'] != $action) return false;
        if (check_admin_referer()) return true;
        die('Invalid call');
    }

     function text($name, $size=20) {
        $value = $this->options[$name];
        if (is_array($value)) $value = implode(',', $value);
        echo '<input name="options[' . $name . ']" type="text" size="' . $size . '" value="';
        echo htmlspecialchars($value);
        echo '"/>';
    }
        
     function checkbox($name) {
        $value = $this->options[$name];
        echo '<input class="panel_checkbox" name="options[' . $name . ']" type="checkbox" value="1"';
        if (!empty($value)) echo ' checked';
        echo '/>';
    }
    
     function textarea($name) {
        echo '<textarea name="options[' . $name . ']">';
        echo htmlspecialchars($this->options[$name]);
        echo '</textarea>'; 
    }

    function select($name, $options) {
        $value = $this->options[$name];

        echo '<select name="options[' . $name . ']">';
        foreach ($options as $key => $label) {
            echo '<option value="' . $key . '"';
            if ($value == $key)
                echo ' selected';
            echo '>' . htmlspecialchars($label) . '&nbsp;&nbsp;</option>';
        }
        echo '</select>';
    }

    function button($action, $label, $message=null) {
        if ($message == null) {
            echo '<input class="button-secondary" type="submit" value="' . $label . '" onclick="this.form.act.value=\'' . $action . '\'"/>';
        } else {
            echo '<input class="button-secondary" type="submit" value="' . $label . '" onclick="this.form.act.value=\'' . $action . '\';return confirm(\'' .
            htmlspecialchars($message) . '\')"/>';
        }
    }

    function init() {
        echo '<script type="text/javascript">
            jQuery(document).ready(function(){
                jQuery("textarea").focus(function() {
                    jQuery("textarea").css("height", "100px");
                    jQuery(this).css("height", "400px");
                });
            });
            </script>
            ';
        echo '<input name="act" type="hidden" value=""/>';
        wp_nonce_field();
    }

    function hidden($name) {
        $value = $this->options[$name];
        echo '<input name="options[' . $name . ']" type="hidden" value="';
        echo htmlspecialchars($value);
        echo '"/>';
    }

    function show_errors() {
        if (empty($this->errors)) return;
        echo '<div class="error"><p>';
        echo $this->errors;
        echo '</p></div>';
    }

    function show_messages() {
        if (empty($this->messages)) return;
        echo '<div class="updated"><p>';
        echo $this->messages;
        echo '</p></div>';
    }

}
?>